
/* IMPORTS */
// icons and labels
const St = imports.gi.St;

// screen and main functionality
const Main = imports.ui.main;
const Tweener = imports.ui.tweener;

// menu items
const PopupMenu = imports.ui.popupMenu;
const Slider = imports.ui.slider;
const Switcher = imports.ui.switcherPopup;

// shutdown functionality
const GnomeSession = imports.misc.gnomeSession;
const Util = imports.misc.util;

// import own scripts
const Extension = imports.misc.extensionUtils.getCurrentExtension();
const Config = Extension.imports.config;
const Timer = Extension.imports.timer;

/* GLOBAL VARIABLES */
let text, submenu, slider, switcher, separator, config, timer;

/* ACTION FUNCTIONS */
// show text box with message
function _showText(textmsg) {
	if(!text) {
		text = new St.Label({ style_class: 'text-label', text: "Hello, world!" });
		Main.uiGroup.add_actor(text);
	}
	text.text = textmsg;
	text.opacity = 255;
	let monitor = Main.layoutManager.primaryMonitor;
	text.set_position(Math.floor(monitor.width / 2 - text.width / 2),
		              Math.floor(monitor.height / 2 - text.height / 2));
	Tweener.addTween(text,
		             { opacity: 0,
		               time: 2,
		               transition: 'easeOutQuad',
		               onComplete: _hideText });
}

function _hideText() {
	Main.uiGroup.remove_actor(text);
	text = null;
}

// update timer value if slider has changed
function _onSliderChanged() {
	timer.timerValue = Math.floor(slider.value * config.maxTimerValue);
	switcher.label.text = timer.timerValue.toString() + ' min';
}

// toggle button starts/stops shutdown timer
function _onToggle() {
	if(switcher.state) {
		timer.startTimer();
		_showText('System will shutdown in ' + timer.timerValue.toString() + ' minutes');
	} else {
		timer.stopTimer();
		_showText('ShutdownTimer stopped');
	}
}

// shutdown the device
function powerOff() {
	Main.overview.hide();
	let session = new GnomeSession.SessionManager();
	session.ShutdownRemote(0);	// shutdown after 60s
	//Util.spawnCommandLine('poweroff');	// shutdown immediately
}

/* EXTENSION MAIN FUNCTIONS */
function init() {
	// initialize timer and config
	config = new Config.Config();
	timer = new Timer.Timer(Math.floor(config.sliderDefaultValue * config.maxTimerValue), powerOff);

	// submenu in status area menu with slider and toggle button
	sliderItem = new PopupMenu.PopupMenuItem('');
	// TODO slider icon
	//let sliderIcon = new St.Icon({ icon_name: 'display-brightness-symbolic', style_class: 'slider-icon' });
	//sliderItem.actor.add(sliderIcon);
	slider = new Slider.Slider(config.sliderDefaultValue);
	slider.connect('value-changed', _onSliderChanged);
	sliderItem.actor.add(slider.actor, { expand: true });

	switcher = new PopupMenu.PopupSwitchMenuItem('');
	switcher.label.text = timer.timerValue.toString() + ' min';
	switcher.connect('toggled', _onToggle);
	
	submenu = new PopupMenu.PopupSubMenuMenuItem('ShutdownTimer', true);
	submenu.icon.icon_name = 'system-shutdown-symbolic';
	submenu.menu.addMenuItem(switcher);
	submenu.menu.addMenuItem(sliderItem);

	separator = new PopupMenu.PopupSeparatorMenuItem();
}

function enable() {
	// add separator line and submenu in status area menu
	let statusMenu = Main.panel.statusArea['aggregateMenu'];
	statusMenu.menu.addMenuItem(separator);
	statusMenu.menu.addMenuItem(submenu);
}

function disable() {
	timer.stopTimer(); // removes timer from Mainloop
	submenu.destroy(); // destroys switcher and sliderItem too
	separator.destroy();
	init();
}
